import CoreMedia


def CMTIMERANGE_IS_VALID(aRange):
    return (
        CMTIME_IS_VALID(aRange.start)
        and CMTIME_IS_VALID(aRange.duration)
        and aRange.duration.epoch == 0
        and aRange.duration.value >= 0
    )


def CMTIMERANGE_IS_INVALID(aRange):
    return not CMTIMERANGE_IS_VALID(aRange)


def CMTIMERANGE_IS_INDEFINITE(aRange):
    return CMTIMERANGE_IS_VALID(aRange) and (
        CMTIME_IS_INDEFINITE(aRange.start) or CMTIME_IS_INDEFINITE(aRange.duration)
    )


def CMTIMERANGE_IS_EMPTY(aRange):
    return CMTIMERANGE_IS_VALID(aRange) and aRange.duration == CoreMedia.kCMTimeZero


def CMTIMEMAPPING_IS_VALID(mapping):
    return CMTIMERANGE_IS_VALID(mapping.target)


def CMTIMEMAPPING_IS_INVALID(mapping):
    return not CMTIMEMAPPING_IS_VALID(mapping)


def CMTIMEMAPPING_IS_EMPTY(mapping):
    return not CMTIME_IS_NUMERIC(mapping.source.start) and CMTIMERANGE_IS_VALID(
        mapping.target
    )


def CMSimpleQueueGetFullness(queue):
    if CoreMedia.CMSimpleQueueGetCapacity(queue):
        return CoreMedia.CMSimpleQueueGetCount(
            queue
        ) / CoreMedia.CMSimpleQueueGetCapacity(queue)
    else:
        return 0.0


def CMTIME_IS_VALID(time):
    return (time.flags & CoreMedia.kCMTimeFlags_Valid) != 0


def CMTIME_IS_INVALID(time):
    return not CMTIME_IS_VALID(time)


def CMTIME_IS_POSITIVE_INFINITY(time):
    return (
        CMTIME_IS_VALID(time)
        and (time.flags & CoreMedia.kCMTimeFlags_PositiveInfinity) != 0
    )


def CMTIME_IS_NEGATIVE_INFINITY(time):
    return (
        CMTIME_IS_VALID(time)
        and (time.flags & CoreMedia.kCMTimeFlags_NegativeInfinity) != 0
    )


def CMTIME_IS_INDEFINITE(time):
    return (
        CMTIME_IS_VALID(time) and (time.flags & CoreMedia.kCMTimeFlags_Indefinite) != 0
    )


def CMTIME_IS_NUMERIC(time):
    return (
        time.flags
        & (CoreMedia.kCMTimeFlags_Valid | CoreMedia.kCMTimeFlags_ImpliedValueFlagsMask)
    ) == CoreMedia.kCMTimeFlags_Valid


def CMTIME_HAS_BEEN_ROUNDED(time):
    return (
        CMTIME_IS_NUMERIC(time)
        and (time.flags & CoreMedia.kCMTimeFlags_HasBeenRounded) != 0
    )



import CoreMedia
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCMFormatDescriptionBridge(TestCase):
    def test_constants(self):
        self.assertEqual(
            CoreMedia.kCMFormatDescriptionBridgeError_InvalidParameter, -12712
        )
        self.assertEqual(
            CoreMedia.kCMFormatDescriptionBridgeError_AllocationFailed, -12713
        )
        self.assertEqual(
            CoreMedia.kCMFormatDescriptionBridgeError_InvalidSerializedSampleDescription,
            -12714,
        )
        self.assertEqual(
            CoreMedia.kCMFormatDescriptionBridgeError_InvalidFormatDescription, -12715
        )
        self.assertEqual(
            CoreMedia.kCMFormatDescriptionBridgeError_IncompatibleFormatDescription,
            -12716,
        )
        self.assertEqual(
            CoreMedia.kCMFormatDescriptionBridgeError_UnsupportedSampleDescriptionFlavor,
            -12717,
        )
        self.assertEqual(CoreMedia.kCMFormatDescriptionBridgeError_InvalidSlice, -12719)

    @min_os_level("10.10")
    def test_constants10_10(self):
        self.assertIsInstance(CoreMedia.kCMImageDescriptionFlavor_QuickTimeMovie, str)
        self.assertIsInstance(CoreMedia.kCMImageDescriptionFlavor_ISOFamily, str)
        self.assertIsInstance(CoreMedia.kCMImageDescriptionFlavor_3GPFamily, str)

        self.assertIsInstance(CoreMedia.kCMSoundDescriptionFlavor_QuickTimeMovie, str)
        self.assertIsInstance(CoreMedia.kCMSoundDescriptionFlavor_QuickTimeMovieV2, str)
        self.assertIsInstance(CoreMedia.kCMSoundDescriptionFlavor_ISOFamily, str)
        self.assertIsInstance(CoreMedia.kCMSoundDescriptionFlavor_3GPFamily, str)

    @min_os_level("10.10")
    def test_functions10_10(self):
        self.assertArgIsIn(
            CoreMedia.CMVideoFormatDescriptionCreateFromBigEndianImageDescriptionData, 1
        )
        self.assertArgSizeInArg(
            CoreMedia.CMVideoFormatDescriptionCreateFromBigEndianImageDescriptionData,
            1,
            2,
        )
        self.assertArgIsOut(
            CoreMedia.CMVideoFormatDescriptionCreateFromBigEndianImageDescriptionData, 5
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMVideoFormatDescriptionCreateFromBigEndianImageDescriptionData, 5
        )

        self.assertArgIsOut(
            CoreMedia.CMVideoFormatDescriptionCreateFromBigEndianImageDescriptionBlockBuffer,
            4,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMVideoFormatDescriptionCreateFromBigEndianImageDescriptionBlockBuffer,
            4,
        )

        self.assertArgIsOut(
            CoreMedia.CMVideoFormatDescriptionCopyAsBigEndianImageDescriptionBlockBuffer,
            4,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMVideoFormatDescriptionCopyAsBigEndianImageDescriptionBlockBuffer,
            4,
        )

        self.assertArgIsInOut(CoreMedia.CMSwapBigEndianImageDescriptionToHost, 0)
        self.assertArgSizeInArg(CoreMedia.CMSwapBigEndianImageDescriptionToHost, 0, 1)

        self.assertArgIsInOut(CoreMedia.CMSwapHostEndianImageDescriptionToBig, 0)
        self.assertArgSizeInArg(CoreMedia.CMSwapHostEndianImageDescriptionToBig, 0, 1)

        self.assertArgIsIn(
            CoreMedia.CMAudioFormatDescriptionCreateFromBigEndianSoundDescriptionData, 1
        )
        self.assertArgSizeInArg(
            CoreMedia.CMAudioFormatDescriptionCreateFromBigEndianSoundDescriptionData,
            1,
            2,
        )
        self.assertArgIsOut(
            CoreMedia.CMAudioFormatDescriptionCreateFromBigEndianSoundDescriptionData, 4
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMAudioFormatDescriptionCreateFromBigEndianSoundDescriptionData, 4
        )

        self.assertArgIsOut(
            CoreMedia.CMAudioFormatDescriptionCreateFromBigEndianSoundDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMAudioFormatDescriptionCreateFromBigEndianSoundDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsOut(
            CoreMedia.CMAudioFormatDescriptionCopyAsBigEndianSoundDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMAudioFormatDescriptionCopyAsBigEndianSoundDescriptionBlockBuffer,
            3,
        )

        self.assertResultIsBOOL(
            CoreMedia.CMDoesBigEndianSoundDescriptionRequireLegacyCBRSampleTableLayout
        )

        self.assertArgIsInOut(CoreMedia.CMSwapBigEndianSoundDescriptionToHost, 0)
        self.assertArgSizeInArg(CoreMedia.CMSwapBigEndianSoundDescriptionToHost, 0, 1)

        self.assertArgIsInOut(CoreMedia.CMSwapHostEndianSoundDescriptionToBig, 0)
        self.assertArgSizeInArg(CoreMedia.CMSwapHostEndianSoundDescriptionToBig, 0, 1)

        self.assertArgIsIn(
            CoreMedia.CMTextFormatDescriptionCreateFromBigEndianTextDescriptionData, 1
        )
        self.assertArgSizeInArg(
            CoreMedia.CMTextFormatDescriptionCreateFromBigEndianTextDescriptionData,
            1,
            2,
        )
        self.assertArgIsOut(
            CoreMedia.CMTextFormatDescriptionCreateFromBigEndianTextDescriptionData, 5
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMTextFormatDescriptionCreateFromBigEndianTextDescriptionData, 5
        )

        self.assertArgIsOut(
            CoreMedia.CMTextFormatDescriptionCreateFromBigEndianTextDescriptionBlockBuffer,
            4,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMTextFormatDescriptionCreateFromBigEndianTextDescriptionBlockBuffer,
            4,
        )

        self.assertArgIsOut(
            CoreMedia.CMTextFormatDescriptionCopyAsBigEndianTextDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMTextFormatDescriptionCopyAsBigEndianTextDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsInOut(CoreMedia.CMSwapBigEndianTextDescriptionToHost, 0)
        self.assertArgSizeInArg(CoreMedia.CMSwapBigEndianTextDescriptionToHost, 0, 1)

        self.assertArgIsInOut(CoreMedia.CMSwapHostEndianTextDescriptionToBig, 0)
        self.assertArgSizeInArg(CoreMedia.CMSwapHostEndianTextDescriptionToBig, 0, 1)

        self.assertArgIsIn(
            CoreMedia.CMClosedCaptionFormatDescriptionCreateFromBigEndianClosedCaptionDescriptionData,
            1,
        )
        self.assertArgSizeInArg(
            CoreMedia.CMClosedCaptionFormatDescriptionCreateFromBigEndianClosedCaptionDescriptionData,
            1,
            2,
        )
        self.assertArgIsOut(
            CoreMedia.CMClosedCaptionFormatDescriptionCreateFromBigEndianClosedCaptionDescriptionData,
            4,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMClosedCaptionFormatDescriptionCreateFromBigEndianClosedCaptionDescriptionData,
            4,
        )

        self.assertArgIsOut(
            CoreMedia.CMClosedCaptionFormatDescriptionCreateFromBigEndianClosedCaptionDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMClosedCaptionFormatDescriptionCreateFromBigEndianClosedCaptionDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsOut(
            CoreMedia.CMClosedCaptionFormatDescriptionCopyAsBigEndianClosedCaptionDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMClosedCaptionFormatDescriptionCopyAsBigEndianClosedCaptionDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsInOut(
            CoreMedia.CMSwapBigEndianClosedCaptionDescriptionToHost, 0
        )
        self.assertArgSizeInArg(
            CoreMedia.CMSwapBigEndianClosedCaptionDescriptionToHost, 0, 1
        )

        self.assertArgIsInOut(
            CoreMedia.CMSwapHostEndianClosedCaptionDescriptionToBig, 0
        )
        self.assertArgSizeInArg(
            CoreMedia.CMSwapHostEndianClosedCaptionDescriptionToBig, 0, 1
        )

        self.assertArgIsIn(
            CoreMedia.CMTimeCodeFormatDescriptionCreateFromBigEndianTimeCodeDescriptionData,
            1,
        )
        self.assertArgSizeInArg(
            CoreMedia.CMTimeCodeFormatDescriptionCreateFromBigEndianTimeCodeDescriptionData,
            1,
            2,
        )
        self.assertArgIsOut(
            CoreMedia.CMTimeCodeFormatDescriptionCreateFromBigEndianTimeCodeDescriptionData,
            4,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMTimeCodeFormatDescriptionCreateFromBigEndianTimeCodeDescriptionData,
            4,
        )

        self.assertArgIsOut(
            CoreMedia.CMTimeCodeFormatDescriptionCreateFromBigEndianTimeCodeDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMTimeCodeFormatDescriptionCreateFromBigEndianTimeCodeDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsOut(
            CoreMedia.CMTimeCodeFormatDescriptionCopyAsBigEndianTimeCodeDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMTimeCodeFormatDescriptionCopyAsBigEndianTimeCodeDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsInOut(CoreMedia.CMSwapBigEndianTimeCodeDescriptionToHost, 0)
        self.assertArgSizeInArg(
            CoreMedia.CMSwapBigEndianTimeCodeDescriptionToHost, 0, 1
        )

        self.assertArgIsInOut(CoreMedia.CMSwapHostEndianTimeCodeDescriptionToBig, 0)
        self.assertArgSizeInArg(
            CoreMedia.CMSwapHostEndianTimeCodeDescriptionToBig, 0, 1
        )

        self.assertArgIsIn(
            CoreMedia.CMMetadataFormatDescriptionCreateFromBigEndianMetadataDescriptionData,
            1,
        )
        self.assertArgSizeInArg(
            CoreMedia.CMMetadataFormatDescriptionCreateFromBigEndianMetadataDescriptionData,
            1,
            2,
        )
        self.assertArgIsOut(
            CoreMedia.CMMetadataFormatDescriptionCreateFromBigEndianMetadataDescriptionData,
            4,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMMetadataFormatDescriptionCreateFromBigEndianMetadataDescriptionData,
            4,
        )

        self.assertArgIsOut(
            CoreMedia.CMMetadataFormatDescriptionCreateFromBigEndianMetadataDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMMetadataFormatDescriptionCreateFromBigEndianMetadataDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsOut(
            CoreMedia.CMMetadataFormatDescriptionCopyAsBigEndianMetadataDescriptionBlockBuffer,
            3,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMMetadataFormatDescriptionCopyAsBigEndianMetadataDescriptionBlockBuffer,
            3,
        )

        self.assertArgIsInOut(CoreMedia.CMSwapBigEndianMetadataDescriptionToHost, 0)
        self.assertArgSizeInArg(
            CoreMedia.CMSwapBigEndianMetadataDescriptionToHost, 0, 1
        )

        self.assertArgIsInOut(CoreMedia.CMSwapHostEndianMetadataDescriptionToBig, 0)
        self.assertArgSizeInArg(
            CoreMedia.CMSwapHostEndianMetadataDescriptionToBig, 0, 1
        )



import CoreMedia
from PyObjCTools.TestSupport import TestCase, min_os_level

CMSampleBufferMakeDataReadyCallback = b"i^{opaqueCMSampleBuffer=}^v"
CMSampleBufferInvalidateHandler = b"v^{opaqueCMSampleBuffer=}"
CMSampleBufferInvalidateCallback = b"v^{opaqueCMSampleBuffer=}Q"


class TestCMSampleBuffer(TestCase):
    def test_constants(self):
        self.assertEqual(CoreMedia.kCMSampleBufferError_AllocationFailed, -12730)
        self.assertEqual(
            CoreMedia.kCMSampleBufferError_RequiredParameterMissing, -12731
        )
        self.assertEqual(CoreMedia.kCMSampleBufferError_AlreadyHasDataBuffer, -12732)
        self.assertEqual(CoreMedia.kCMSampleBufferError_BufferNotReady, -12733)
        self.assertEqual(CoreMedia.kCMSampleBufferError_SampleIndexOutOfRange, -12734)
        self.assertEqual(CoreMedia.kCMSampleBufferError_BufferHasNoSampleSizes, -12735)
        self.assertEqual(
            CoreMedia.kCMSampleBufferError_BufferHasNoSampleTimingInfo, -12736
        )
        self.assertEqual(CoreMedia.kCMSampleBufferError_ArrayTooSmall, -12737)
        self.assertEqual(CoreMedia.kCMSampleBufferError_InvalidEntryCount, -12738)
        self.assertEqual(CoreMedia.kCMSampleBufferError_CannotSubdivide, -12739)
        self.assertEqual(CoreMedia.kCMSampleBufferError_SampleTimingInfoInvalid, -12740)
        self.assertEqual(
            CoreMedia.kCMSampleBufferError_InvalidMediaTypeForOperation, -12741
        )
        self.assertEqual(CoreMedia.kCMSampleBufferError_InvalidSampleData, -12742)
        self.assertEqual(CoreMedia.kCMSampleBufferError_InvalidMediaFormat, -12743)
        self.assertEqual(CoreMedia.kCMSampleBufferError_Invalidated, -12744)
        self.assertEqual(CoreMedia.kCMSampleBufferError_DataFailed, -16750)
        self.assertEqual(CoreMedia.kCMSampleBufferError_DataCanceled, -16751)

        self.assertEqual(
            CoreMedia.kCMSampleBufferFlag_AudioBufferList_Assure16ByteAlignment, 1 << 0
        )

        self.assertIsInstance(
            CoreMedia.kCMTimingInfoInvalid, CoreMedia.CMSampleTimingInfo
        )

        self.assertIsInstance(
            CoreMedia.kCMSampleBufferNotification_DataBecameReady, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConduitNotification_InhibitOutputUntil, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConduitNotificationParameter_ResumeTag, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConduitNotification_ResetOutput, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConduitNotification_UpcomingOutputPTSRangeChanged,
            str,
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConduitNotificationParameter_UpcomingOutputPTSRangeMayOverlapQueuedOutputPTSRange,
            str,
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConduitNotificationParameter_MinUpcomingOutputPTS,
            str,
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConsumerNotification_BufferConsumed, str
        )

        self.assertIsInstance(CoreMedia.kCMSampleAttachmentKey_NotSync, str)
        self.assertIsInstance(CoreMedia.kCMSampleAttachmentKey_PartialSync, str)
        self.assertIsInstance(CoreMedia.kCMSampleAttachmentKey_HasRedundantCoding, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleAttachmentKey_IsDependedOnByOthers, str
        )
        self.assertIsInstance(CoreMedia.kCMSampleAttachmentKey_DependsOnOthers, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleAttachmentKey_EarlierDisplayTimesAllowed, str
        )
        self.assertIsInstance(CoreMedia.kCMSampleAttachmentKey_DisplayImmediately, str)
        self.assertIsInstance(CoreMedia.kCMSampleAttachmentKey_DoNotDisplay, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_ResetDecoderBeforeDecoding, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_DrainAfterDecoding, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_PostNotificationWhenConsumed, str
        )
        self.assertIsInstance(CoreMedia.kCMSampleBufferAttachmentKey_ResumeOutput, str)
        self.assertIsInstance(CoreMedia.kCMSampleBufferAttachmentKey_TransitionID, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_TrimDurationAtStart, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_TrimDurationAtEnd, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_SpeedMultiplier, str
        )
        self.assertIsInstance(CoreMedia.kCMSampleBufferAttachmentKey_Reverse, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_FillDiscontinuitiesWithSilence, str
        )
        self.assertIsInstance(CoreMedia.kCMSampleBufferAttachmentKey_EmptyMedia, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_PermanentEmptyMedia, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_DisplayEmptyMediaImmediately, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_EndsPreviousSampleDuration, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_SampleReferenceURL, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_SampleReferenceByteOffset, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferAttachmentKey_GradualDecoderRefresh, str
        )

    @min_os_level("10.8")
    def test_constants10_8(self):
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferConduitNotificationParameter_MaxUpcomingOutputPTS,
            str,
        )

    @min_os_level("10.10")
    def test_constants10_10(self):
        self.assertIsInstance(CoreMedia.kCMSampleBufferNotification_DataFailed, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleBufferNotificationParameter_OSStatus, str
        )
        self.assertIsInstance(CoreMedia.kCMSampleBufferAttachmentKey_ForceKeyFrame, str)

    @min_os_level("10.13")
    def test_constants10_13(self):
        self.assertIsInstance(
            CoreMedia.kCMSampleAttachmentKey_HEVCTemporalLevelInfo, str
        )
        self.assertIsInstance(CoreMedia.kCMHEVCTemporalLevelInfoKey_TemporalLevel, str)
        self.assertIsInstance(CoreMedia.kCMHEVCTemporalLevelInfoKey_ProfileSpace, str)
        self.assertIsInstance(CoreMedia.kCMHEVCTemporalLevelInfoKey_TierFlag, str)
        self.assertIsInstance(CoreMedia.kCMHEVCTemporalLevelInfoKey_ProfileIndex, str)
        self.assertIsInstance(
            CoreMedia.kCMHEVCTemporalLevelInfoKey_ProfileCompatibilityFlags, str
        )
        self.assertIsInstance(
            CoreMedia.kCMHEVCTemporalLevelInfoKey_ConstraintIndicatorFlags, str
        )
        self.assertIsInstance(CoreMedia.kCMHEVCTemporalLevelInfoKey_LevelIndex, str)
        self.assertIsInstance(
            CoreMedia.kCMSampleAttachmentKey_HEVCTemporalSubLayerAccess, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleAttachmentKey_HEVCStepwiseTemporalSubLayerAccess, str
        )
        self.assertIsInstance(
            CoreMedia.kCMSampleAttachmentKey_HEVCSyncSampleNALUnitType, str
        )

    @min_os_level("10.15")
    def test_constants10_15(self):
        self.assertIsInstance(
            CoreMedia.kCMSampleAttachmentKey_AudioIndependentSampleDecoderRefreshCount,
            str,
        )

    def test_types(self):
        self.assertIsCFType(CoreMedia.CMSampleBufferRef)

    def test_structs(self):
        v = CoreMedia.CMSampleTimingInfo()
        self.assertEqual(v.duration, CoreMedia.CMTime())
        self.assertEqual(v.presentationTimeStamp, CoreMedia.CMTime())
        self.assertEqual(v.decodeTimeStamp, CoreMedia.CMTime())

    def test_functions(self):
        self.assertArgIsBOOL(CoreMedia.CMSampleBufferCreate, 2)
        self.assertArgIsFunction(
            CoreMedia.CMSampleBufferCreate, 3, CMSampleBufferMakeDataReadyCallback, True
        )
        self.assertArgIsIn(CoreMedia.CMSampleBufferCreate, 8)
        self.assertArgSizeInArg(CoreMedia.CMSampleBufferCreate, 8, 7)
        self.assertArgIsIn(CoreMedia.CMSampleBufferCreate, 10)
        self.assertArgSizeInArg(CoreMedia.CMSampleBufferCreate, 10, 9)
        self.assertArgIsOut(CoreMedia.CMSampleBufferCreate, 11)
        self.assertArgIsCFRetained(CoreMedia.CMSampleBufferCreate, 11)

        self.assertArgIsBOOL(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptions, 2
        )
        self.assertArgIsFunction(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptions,
            3,
            CMSampleBufferMakeDataReadyCallback,
            True,
        )
        self.assertArgIsIn(CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptions, 8)
        self.assertArgSizeInArg(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptions, 8, 6
        )
        self.assertArgIsOut(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptions, 9
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptions, 9
        )

        self.assertArgIsBOOL(CoreMedia.CMSampleBufferCreateForImageBuffer, 2)
        self.assertArgIsFunction(
            CoreMedia.CMSampleBufferCreateForImageBuffer,
            3,
            CMSampleBufferMakeDataReadyCallback,
            True,
        )
        self.assertArgIsIn(CoreMedia.CMSampleBufferCreateForImageBuffer, 6)
        self.assertArgIsOut(CoreMedia.CMSampleBufferCreateForImageBuffer, 7)
        self.assertArgIsCFRetained(CoreMedia.CMSampleBufferCreateForImageBuffer, 7)

        self.assertArgIsOut(CoreMedia.CMSampleBufferCreateCopy, 2)
        self.assertArgIsCFRetained(CoreMedia.CMSampleBufferCreateCopy, 2)

        self.assertArgIsIn(CoreMedia.CMSampleBufferCreateCopyWithNewTiming, 3)
        self.assertArgSizeInArg(CoreMedia.CMSampleBufferCreateCopyWithNewTiming, 3, 2)
        self.assertArgIsOut(CoreMedia.CMSampleBufferCreateCopyWithNewTiming, 4)
        self.assertArgIsCFRetained(CoreMedia.CMSampleBufferCreateCopyWithNewTiming, 4)

        self.assertArgIsOut(CoreMedia.CMSampleBufferCopySampleBufferForRange, 3)
        self.assertArgIsCFRetained(CoreMedia.CMSampleBufferCopySampleBufferForRange, 3)

        CoreMedia.CMSampleBufferGetTypeID
        CoreMedia.CMSampleBufferSetDataBuffer
        CoreMedia.CMSampleBufferGetDataBuffer
        CoreMedia.CMSampleBufferGetImageBuffer
        CoreMedia.CMSampleBufferSetDataBufferFromAudioBufferList

        self.assertArgIsOut(
            CoreMedia.CMSampleBufferGetAudioBufferListWithRetainedBlockBuffer, 1
        )
        self.assertArgIsOut(
            CoreMedia.CMSampleBufferGetAudioBufferListWithRetainedBlockBuffer, 7
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMSampleBufferGetAudioBufferListWithRetainedBlockBuffer, 7
        )

        self.assertArgIsOut(CoreMedia.CMSampleBufferGetAudioStreamPacketDescriptions, 2)
        self.assertArgSizeInArg(
            CoreMedia.CMSampleBufferGetAudioStreamPacketDescriptions, 2, 1
        )
        self.assertArgIsOut(CoreMedia.CMSampleBufferGetAudioStreamPacketDescriptions, 3)

        self.assertFalse(
            hasattr(CoreMedia, "CMSampleBufferGetAudioStreamPacketDescriptionsPtr")
        )

        CoreMedia.CMSampleBufferCopyPCMDataIntoAudioBufferList
        CoreMedia.CMSampleBufferSetDataReady

        self.assertResultIsBOOL(CoreMedia.CMSampleBufferDataIsReady)

        CoreMedia.CMSampleBufferMakeDataReady
        CoreMedia.CMSampleBufferTrackDataReadiness
        CoreMedia.CMSampleBufferInvalidate

        self.assertArgIsFunction(
            CoreMedia.CMSampleBufferSetInvalidateCallback,
            1,
            CMSampleBufferInvalidateCallback,
            True,
        )

        self.assertResultIsBOOL(CoreMedia.CMSampleBufferIsValid)

        CoreMedia.CMSampleBufferGetNumSamples
        CoreMedia.CMSampleBufferGetDuration
        CoreMedia.CMSampleBufferGetPresentationTimeStamp
        CoreMedia.CMSampleBufferGetDecodeTimeStamp
        CoreMedia.CMSampleBufferGetOutputDuration
        CoreMedia.CMSampleBufferGetOutputPresentationTimeStamp
        CoreMedia.CMSampleBufferSetOutputPresentationTimeStamp
        CoreMedia.CMSampleBufferGetOutputDecodeTimeStamp

        self.assertArgIsOut(CoreMedia.CMSampleBufferGetSampleTimingInfoArray, 2)
        self.assertArgSizeInArg(CoreMedia.CMSampleBufferGetSampleTimingInfoArray, 2, 1)
        self.assertArgIsOut(CoreMedia.CMSampleBufferGetSampleTimingInfoArray, 3)

        self.assertArgIsOut(CoreMedia.CMSampleBufferGetOutputSampleTimingInfoArray, 2)
        self.assertArgSizeInArg(
            CoreMedia.CMSampleBufferGetOutputSampleTimingInfoArray, 2, 1
        )
        self.assertArgIsOut(CoreMedia.CMSampleBufferGetOutputSampleTimingInfoArray, 3)

        self.assertArgIsOut(CoreMedia.CMSampleBufferGetSampleTimingInfo, 2)

        self.assertArgIsOut(CoreMedia.CMSampleBufferGetSampleSizeArray, 2)
        self.assertArgSizeInArg(CoreMedia.CMSampleBufferGetSampleSizeArray, 2, 1)
        self.assertArgIsOut(CoreMedia.CMSampleBufferGetSampleSizeArray, 3)

        CoreMedia.CMSampleBufferGetSampleSize
        CoreMedia.CMSampleBufferGetTotalSampleSize
        CoreMedia.CMSampleBufferGetFormatDescription

        self.assertArgIsBOOL(CoreMedia.CMSampleBufferGetSampleAttachmentsArray, 1)

        self.assertArgIsFunction(
            CoreMedia.CMSampleBufferCallForEachSample,
            1,
            b"i^{opaqueCMSampleBuffer=}q^v",
            False,
        )

    @min_os_level("10.10")
    def test_functions10_10(self):
        self.assertArgIsIn(CoreMedia.CMSampleBufferCreateReady, 5)
        self.assertArgSizeInArg(CoreMedia.CMSampleBufferCreateReady, 5, 4)
        self.assertArgIsIn(CoreMedia.CMSampleBufferCreateReady, 7)
        self.assertArgSizeInArg(CoreMedia.CMSampleBufferCreateReady, 7, 6)
        self.assertArgIsOut(CoreMedia.CMSampleBufferCreateReady, 8)
        self.assertArgIsCFRetained(CoreMedia.CMSampleBufferCreateReady, 8)

        self.assertArgIsIn(
            CoreMedia.CMAudioSampleBufferCreateReadyWithPacketDescriptions, 5
        )
        self.assertArgSizeInArg(
            CoreMedia.CMAudioSampleBufferCreateReadyWithPacketDescriptions, 5, 3
        )
        self.assertArgIsOut(
            CoreMedia.CMAudioSampleBufferCreateReadyWithPacketDescriptions, 6
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMAudioSampleBufferCreateReadyWithPacketDescriptions, 6
        )

        self.assertArgIsIn(CoreMedia.CMSampleBufferCreateReadyWithImageBuffer, 3)
        self.assertArgIsOut(CoreMedia.CMSampleBufferCreateReadyWithImageBuffer, 4)
        self.assertArgIsCFRetained(
            CoreMedia.CMSampleBufferCreateReadyWithImageBuffer, 4
        )

        CoreMedia.CMSampleBufferSetDataFailed

        self.assertResultIsBOOL(CoreMedia.CMSampleBufferHasDataFailed)
        self.assertArgIsOut(CoreMedia.CMSampleBufferHasDataFailed, 1)

        self.assertArgIsBlock(
            CoreMedia.CMSampleBufferSetInvalidateHandler,
            1,
            CMSampleBufferInvalidateHandler,
        )

        self.assertArgIsBlock(
            CoreMedia.CMSampleBufferCallBlockForEachSample,
            1,
            b"v^{opaqueCMSampleBuffer=}q",
        )

    @min_os_level("10.14.4")
    def test_functions_10_14_4(self):
        self.assertArgIsBOOL(CoreMedia.CMSampleBufferCreateWithMakeDataReadyHandler, 2)
        self.assertArgIsIn(CoreMedia.CMSampleBufferCreateWithMakeDataReadyHandler, 6)
        self.assertArgSizeInArg(
            CoreMedia.CMSampleBufferCreateWithMakeDataReadyHandler, 6, 5
        )
        self.assertArgIsOut(CoreMedia.CMSampleBufferCreateWithMakeDataReadyHandler, 7)
        self.assertArgIsCFRetained(
            CoreMedia.CMSampleBufferCreateWithMakeDataReadyHandler, 7
        )
        self.assertArgIsBlock(
            CoreMedia.CMSampleBufferCreateWithMakeDataReadyHandler,
            8,
            b"i^{OpaqueCMSampleBuffer=}",
        )

        self.assertArgIsBOOL(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptionsAndMakeDataReadyHandler,
            2,
        )
        self.assertArgIsIn(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptionsAndMakeDataReadyHandler,
            6,
        )
        self.assertArgSizeInArg(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptionsAndMakeDataReadyHandler,
            6,
            5,
        )
        self.assertArgIsOut(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptionsAndMakeDataReadyHandler,
            7,
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptionsAndMakeDataReadyHandler,
            7,
        )
        self.assertArgIsBlock(
            CoreMedia.CMAudioSampleBufferCreateWithPacketDescriptionsAndMakeDataReadyHandler,
            8,
            b"i^{OpaqueCMSampleBuffer=}",
        )

        self.assertArgIsBOOL(
            CoreMedia.CMSampleBufferCreateForImageBufferWithMakeDataReadyHandler, 2
        )
        self.assertArgIsOut(
            CoreMedia.CMSampleBufferCreateForImageBufferWithMakeDataReadyHandler, 5
        )
        self.assertArgIsCFRetained(
            CoreMedia.CMSampleBufferCreateForImageBufferWithMakeDataReadyHandler, 5
        )
        self.assertArgIsBlock(
            CoreMedia.CMSampleBufferCreateForImageBufferWithMakeDataReadyHandler,
            6,
            b"i^{OpaqueCMSampleBuffer=}",
        )



import CoreMedia
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCMSync(TestCase):
    def test_types(self):
        self.assertIsCFType(CoreMedia.CMClockRef)
        self.assertIsCFType(CoreMedia.CMTimebaseRef)

    def test_constants(self):
        self.assertEqual(CoreMedia.kCMClockError_MissingRequiredParameter, -12745)
        self.assertEqual(CoreMedia.kCMClockError_InvalidParameter, -12746)
        self.assertEqual(CoreMedia.kCMClockError_AllocationFailed, -12747)
        self.assertEqual(CoreMedia.kCMClockError_UnsupportedOperation, -12756)

        self.assertEqual(CoreMedia.kCMTimebaseError_MissingRequiredParameter, -12748)
        self.assertEqual(CoreMedia.kCMTimebaseError_InvalidParameter, -12749)
        self.assertEqual(CoreMedia.kCMTimebaseError_AllocationFailed, -12750)
        self.assertEqual(CoreMedia.kCMTimebaseError_TimerIntervalTooShort, -12751)
        self.assertEqual(CoreMedia.kCMTimebaseError_ReadOnly, -12757)

        self.assertEqual(CoreMedia.kCMSyncError_MissingRequiredParameter, -12752)
        self.assertEqual(CoreMedia.kCMSyncError_InvalidParameter, -12753)
        self.assertEqual(CoreMedia.kCMSyncError_AllocationFailed, -12754)
        self.assertEqual(CoreMedia.kCMSyncError_RateMustBeNonZero, -12755)

        self.assertEqual(
            CoreMedia.kCMTimebaseVeryLongCFTimeInterval,
            256.0 * 365.0 * 24.0 * 60.0 * 60.0,
        )
        self.assertEqual(
            CoreMedia.kCMTimebaseFarFutureCFAbsoluteTime,
            CoreMedia.kCMTimebaseVeryLongCFTimeInterval,
        )

    @min_os_level("10.8")
    def test_constants10_8(self):
        self.assertIsInstance(
            CoreMedia.kCMTimebaseNotification_EffectiveRateChanged, str
        )
        self.assertIsInstance(CoreMedia.kCMTimebaseNotification_TimeJumped, str)

    @min_os_level("10.9")
    def test_constants10_9(self):
        self.assertIsInstance(CoreMedia.kCMTimebaseNotificationKey_EventTime, str)

    @min_os_level("10.8")
    def test_functions(self):
        CoreMedia.CMClockGetTypeID
        CoreMedia.CMClockGetHostTimeClock
        CoreMedia.CMClockConvertHostTimeToSystemUnits
        CoreMedia.CMClockMakeHostTimeFromSystemUnits
        CoreMedia.CMClockGetTime

        self.assertArgIsOut(CoreMedia.CMClockGetAnchorTime, 1)
        self.assertArgIsOut(CoreMedia.CMClockGetAnchorTime, 2)

        CoreMedia.CMClockMightDrift
        CoreMedia.CMClockInvalidate
        CoreMedia.CMTimebaseGetTypeID

        self.assertArgIsOut(CoreMedia.CMTimebaseCreateWithMasterClock, 2)
        self.assertArgIsCFRetained(CoreMedia.CMTimebaseCreateWithMasterClock, 2)

        self.assertArgIsOut(CoreMedia.CMTimebaseCreateWithMasterTimebase, 2)
        self.assertArgIsCFRetained(CoreMedia.CMTimebaseCreateWithMasterTimebase, 2)

        CoreMedia.CMTimebaseGetMasterTimebase
        CoreMedia.CMTimebaseGetMasterClock
        CoreMedia.CMTimebaseGetMaster
        CoreMedia.CMTimebaseGetUltimateMasterClock
        CoreMedia.CMTimebaseGetTime
        CoreMedia.CMTimebaseGetTimeWithTimeScale
        CoreMedia.CMTimebaseSetTime
        CoreMedia.CMTimebaseSetAnchorTime
        CoreMedia.CMTimebaseGetRate
        CoreMedia.CMTimebaseGetTimeAndRate
        CoreMedia.CMTimebaseSetRate
        CoreMedia.CMTimebaseSetRateAndAnchorTime
        CoreMedia.CMTimebaseGetEffectiveRate
        CoreMedia.CMTimebaseAddTimer
        CoreMedia.CMTimebaseRemoveTimer
        CoreMedia.CMTimebaseSetTimerNextFireTime
        CoreMedia.CMTimebaseSetTimerToFireImmediately
        CoreMedia.CMTimebaseAddTimerDispatchSource
        CoreMedia.CMTimebaseRemoveTimerDispatchSource
        CoreMedia.CMTimebaseSetTimerDispatchSourceNextFireTime
        CoreMedia.CMTimebaseSetTimerDispatchSourceToFireImmediately
        CoreMedia.CMSyncGetRelativeRate

        self.assertArgIsOut(CoreMedia.CMSyncGetRelativeRateAndAnchorTime, 2)
        self.assertArgIsOut(CoreMedia.CMSyncGetRelativeRateAndAnchorTime, 3)
        self.assertArgIsOut(CoreMedia.CMSyncGetRelativeRateAndAnchorTime, 4)

        CoreMedia.CMSyncConvertTime
        CoreMedia.CMSyncMightDrift
        CoreMedia.CMSyncGetTime
        CoreMedia.CMTimebaseNotificationBarrier

        CoreMedia.CMTimebaseSetMasterClock
        CoreMedia.CMTimebaseSetMasterTimebase

    @min_os_level("10.11")
    def test_functions10_11(self):
        self.assertResultIsCFRetained(CoreMedia.CMTimebaseCopyMasterTimebase)
        self.assertResultIsCFRetained(CoreMedia.CMTimebaseCopyMasterClock)
        self.assertResultIsCFRetained(CoreMedia.CMTimebaseCopyMaster)
        self.assertResultIsCFRetained(CoreMedia.CMTimebaseCopyUltimateMasterClock)



import CoreMedia
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestCMBufferQueue(TestCase):
    def test_constants(self):
        self.assertEqual(CoreMedia.kCMBufferQueueError_AllocationFailed, -12760)
        self.assertEqual(CoreMedia.kCMBufferQueueError_RequiredParameterMissing, -12761)
        self.assertEqual(
            CoreMedia.kCMBufferQueueError_InvalidCMBufferCallbacksStruct, -12762
        )
        self.assertEqual(CoreMedia.kCMBufferQueueError_EnqueueAfterEndOfData, -12763)
        self.assertEqual(CoreMedia.kCMBufferQueueError_QueueIsFull, -12764)
        self.assertEqual(CoreMedia.kCMBufferQueueError_BadTriggerDuration, -12765)
        self.assertEqual(
            CoreMedia.kCMBufferQueueError_CannotModifyQueueFromTriggerCallback, -12766
        )
        self.assertEqual(CoreMedia.kCMBufferQueueError_InvalidTriggerCondition, -12767)
        self.assertEqual(CoreMedia.kCMBufferQueueError_InvalidTriggerToken, -12768)
        self.assertEqual(CoreMedia.kCMBufferQueueError_InvalidBuffer, -12769)

        self.assertEqual(CoreMedia.kCMBufferQueueTrigger_WhenDurationBecomesLessThan, 1)
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenDurationBecomesLessThanOrEqualTo, 2
        )
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenDurationBecomesGreaterThan, 3
        )
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenDurationBecomesGreaterThanOrEqualTo, 4
        )
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenMinPresentationTimeStampChanges, 5
        )
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenMaxPresentationTimeStampChanges, 6
        )
        self.assertEqual(CoreMedia.kCMBufferQueueTrigger_WhenDataBecomesReady, 7)
        self.assertEqual(CoreMedia.kCMBufferQueueTrigger_WhenEndOfDataReached, 8)
        self.assertEqual(CoreMedia.kCMBufferQueueTrigger_WhenReset, 9)
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenBufferCountBecomesLessThan, 10
        )
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenBufferCountBecomesGreaterThan, 11
        )
        self.assertEqual(
            CoreMedia.kCMBufferQueueTrigger_WhenDurationBecomesGreaterThanOrEqualToAndBufferCountBecomesGreaterThan,
            12,
        )

    def test_cftypes(self):
        self.assertIsCFType(CoreMedia.CMBufferQueueRef)

    def test_opaque(self):
        self.assertIsOpaquePointer(CoreMedia.CMBufferQueueTriggerToken)

    @expectedFailure
    def test_functions_manual(self):
        self.fail("CMBufferQueueGetCallbacksForUnsortedSampleBuffers")
        self.fail("CMBufferQueueGetCallbacksForSampleBuffersSortedByOutputPTS")
        self.fail("CMBufferQueueCreate")

    def test_functions(self):
        CoreMedia.CMBufferQueueGetTypeID
        CoreMedia.CMBufferQueueEnqueue

        self.assertResultIsCFRetained(CoreMedia.CMBufferQueueDequeueAndRetain)
        self.assertResultIsCFRetained(
            CoreMedia.CMBufferQueueDequeueIfDataReadyAndRetain
        )

        CoreMedia.CMBufferQueueGetHead

        self.assertResultIsBOOL(CoreMedia.CMBufferQueueIsEmpty)

        CoreMedia.CMBufferQueueMarkEndOfData

        self.assertResultIsBOOL(CoreMedia.CMBufferQueueContainsEndOfData)
        self.assertResultIsBOOL(CoreMedia.CMBufferQueueIsAtEndOfData)

        CoreMedia.CMBufferQueueReset

        self.assertArgIsFunction(
            CoreMedia.CMBufferQueueResetWithCallback, 1, b"v@^v", False
        )

        CoreMedia.CMBufferQueueGetBufferCount
        CoreMedia.CMBufferQueueGetDuration
        CoreMedia.CMBufferQueueGetMinDecodeTimeStamp
        CoreMedia.CMBufferQueueGetFirstDecodeTimeStamp
        CoreMedia.CMBufferQueueGetMinPresentationTimeStamp
        CoreMedia.CMBufferQueueGetFirstPresentationTimeStamp
        CoreMedia.CMBufferQueueGetMaxPresentationTimeStamp
        CoreMedia.CMBufferQueueGetEndPresentationTimeStamp

        self.assertArgIsFunction(
            CoreMedia.CMBufferQueueInstallTrigger,
            1,
            b"v^v^{opaqueCMBufferQueueTriggerToken=}",
            True,
        )
        self.assertArgIsOut(CoreMedia.CMBufferQueueInstallTrigger, 5)

        self.assertArgIsFunction(
            CoreMedia.CMBufferQueueInstallTriggerWithIntegerThreshold,
            1,
            b"v^v^{opaqueCMBufferQueueTriggerToken=}",
            True,
        )
        self.assertArgIsOut(
            CoreMedia.CMBufferQueueInstallTriggerWithIntegerThreshold, 5
        )

        CoreMedia.CMBufferQueueRemoveTrigger

        self.assertResultIsBOOL(CoreMedia.CMBufferQueueTestTrigger)

        self.assertArgIsFunction(
            CoreMedia.CMBufferQueueCallForEachBuffer, 1, b"i@^v", True
        )

        self.assertArgIsFunction(
            CoreMedia.CMBufferQueueSetValidationCallback,
            1,
            b"i^{opaqueCMBufferQueue=}@^v",
            True,
        )

    @min_os_level("10.10")
    def test_functions10_10(self):
        CoreMedia.CMBufferQueueGetTotalSize

    @min_os_level("10.14.4")
    def test_functions10_14_4(self):
        self.assertNotHasAttr(CoreMedia, "CMBufferQueueCreateWithHandlers")

        self.assertArgIsOut(CoreMedia.CMBufferQueueInstallTriggerHandler, 3)
        self.assertArgIsBlock(
            CoreMedia.CMBufferQueueInstallTriggerHandler,
            4,
            b"v^{opaqueCMBufferQueueTriggerToken=}",
        )

        self.assertArgIsOut(
            CoreMedia.CMBufferQueueInstallTriggerHandlerWithIntegerThreshold, 3
        )
        self.assertArgIsBlock(
            CoreMedia.CMBufferQueueInstallTriggerHandlerWithIntegerThreshold,
            4,
            b"v^{opaqueCMBufferQueueTriggerToken=}",
        )

        self.assertArgIsBlock(CoreMedia.CMBufferQueueSetValidationHandler, 1, b"v@@")



import CoreMedia
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCMTime(TestCase):
    def test_constants(self):
        self.assertEqual(CoreMedia.kCMTimeMaxTimescale, 0x7FFFFFFF)

        self.assertEqual(CoreMedia.kCMTimeFlags_Valid, 1 << 0)
        self.assertEqual(CoreMedia.kCMTimeFlags_HasBeenRounded, 1 << 1)
        self.assertEqual(CoreMedia.kCMTimeFlags_PositiveInfinity, 1 << 2)
        self.assertEqual(CoreMedia.kCMTimeFlags_NegativeInfinity, 1 << 3)
        self.assertEqual(CoreMedia.kCMTimeFlags_Indefinite, 1 << 4)
        self.assertEqual(
            CoreMedia.kCMTimeFlags_ImpliedValueFlagsMask,
            CoreMedia.kCMTimeFlags_PositiveInfinity
            | CoreMedia.kCMTimeFlags_NegativeInfinity
            | CoreMedia.kCMTimeFlags_Indefinite,
        )

        self.assertEqual(CoreMedia.kCMTimeRoundingMethod_RoundHalfAwayFromZero, 1)
        self.assertEqual(CoreMedia.kCMTimeRoundingMethod_RoundTowardZero, 2)
        self.assertEqual(CoreMedia.kCMTimeRoundingMethod_RoundAwayFromZero, 3)
        self.assertEqual(CoreMedia.kCMTimeRoundingMethod_QuickTime, 4)
        self.assertEqual(CoreMedia.kCMTimeRoundingMethod_RoundTowardPositiveInfinity, 5)
        self.assertEqual(CoreMedia.kCMTimeRoundingMethod_RoundTowardNegativeInfinity, 6)
        self.assertEqual(
            CoreMedia.kCMTimeRoundingMethod_Default,
            CoreMedia.kCMTimeRoundingMethod_RoundHalfAwayFromZero,
        )

    def test_structs(self):
        v = CoreMedia.CMTime()
        self.assertEqual(v.value, 0)
        self.assertEqual(v.timescale, 0)
        self.assertEqual(v.flags, 0)
        self.assertEqual(v.epoch, 0)

    def test_contants(self):
        self.assertIsInstance(CoreMedia.kCMTimeInvalid, CoreMedia.CMTime)
        self.assertIsInstance(CoreMedia.kCMTimeIndefinite, CoreMedia.CMTime)
        self.assertIsInstance(CoreMedia.kCMTimePositiveInfinity, CoreMedia.CMTime)
        self.assertIsInstance(CoreMedia.kCMTimeNegativeInfinity, CoreMedia.CMTime)
        self.assertIsInstance(CoreMedia.kCMTimeZero, CoreMedia.CMTime)

        self.assertIsInstance(CoreMedia.kCMTimeValueKey, str)
        self.assertIsInstance(CoreMedia.kCMTimeScaleKey, str)
        self.assertIsInstance(CoreMedia.kCMTimeEpochKey, str)
        self.assertIsInstance(CoreMedia.kCMTimeFlagsKey, str)

    def test_functions(self):
        CoreMedia.CMTIME_IS_VALID
        CoreMedia.CMTIME_IS_INVALID
        CoreMedia.CMTIME_IS_POSITIVE_INFINITY
        CoreMedia.CMTIME_IS_NEGATIVE_INFINITY
        CoreMedia.CMTIME_IS_INDEFINITE
        CoreMedia.CMTIME_IS_NUMERIC
        CoreMedia.CMTIME_HAS_BEEN_ROUNDED

        CoreMedia.CMTimeMake
        CoreMedia.CMTimeMakeWithEpoch
        CoreMedia.CMTimeMakeWithSeconds
        CoreMedia.CMTimeGetSeconds
        CoreMedia.CMTimeConvertScale
        CoreMedia.CMTimeAdd
        CoreMedia.CMTimeSubtract
        CoreMedia.CMTimeMultiply
        CoreMedia.CMTimeMultiplyByFloat64
        CoreMedia.CMTimeCompare

        self.assertFalse(hasattr(CoreMedia, "CMTIME_COMPARE_INLINE"))

        CoreMedia.CMTimeMinimum
        CoreMedia.CMTimeMaximum
        CoreMedia.CMTimeAbsoluteValue

        self.assertResultIsCFRetained(CoreMedia.CMTimeCopyAsDictionary)

        CoreMedia.CMTimeMakeFromDictionary

        self.assertResultIsCFRetained(CoreMedia.CMTimeCopyDescription)

        CoreMedia.CMTimeShow

    @min_os_level("10.10")
    def test_functions10_10(self):
        CoreMedia.CMTimeMultiplyByRatio



import CoreMedia
from PyObjCTools.TestSupport import TestCase


class TestCMSimpleQueue(TestCase):
    def test_constants(self):
        self.assertEqual(CoreMedia.kCMSimpleQueueError_AllocationFailed, -12770)
        self.assertEqual(CoreMedia.kCMSimpleQueueError_RequiredParameterMissing, -12771)
        self.assertEqual(CoreMedia.kCMSimpleQueueError_ParameterOutOfRange, -12772)
        self.assertEqual(CoreMedia.kCMSimpleQueueError_QueueIsFull, -12773)

    def test_types(self):
        self.assertIsCFType(CoreMedia.CMSimpleQueueRef)

    def test_functions(self):
        CoreMedia.CMSimpleQueueGetTypeID

        self.assertArgIsOut(CoreMedia.CMSimpleQueueCreate, 2)
        self.assertArgIsCFRetained(CoreMedia.CMSimpleQueueCreate, 2)

        self.assertArgHasType(CoreMedia.CMSimpleQueueEnqueue, 1, b"@")

        CoreMedia.CMSimpleQueueDequeue
        CoreMedia.CMSimpleQueueGetHead
        CoreMedia.CMSimpleQueueReset
        CoreMedia.CMSimpleQueueGetCapacity
        CoreMedia.CMSimpleQueueGetCount

        CoreMedia.CMSimpleQueueGetFullness



"""
Python mapping for the CoreMedia framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import sys

import Foundation
import objc
from CoreMedia import _CoreMedia, _metadata

sys.modules["CoreMedia"] = mod = objc.ObjCLazyModule(
    "CoreMedia",
    "com.apple.CoreMedia",
    objc.pathForFramework("/System/Library/Frameworks/CoreMedia.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (_CoreMedia, Foundation),
)

import sys  # isort: ignore  # noqa: E402
from CoreMedia import _macros  # isort: ignore  # noqa: E402

for nm in dir(_macros):
    if nm == "CoreMedia":
        continue
    setattr(mod, nm, getattr(_macros, nm))


del sys.modules["CoreMedia._metadata"]



import CoreMedia
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCMTextMarkup(TestCase):
    @min_os_level("10.9")
    def test_constants10_9(self):
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_ForegroundColorARGB, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_BackgroundColorARGB, str)
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupAttribute_CharacterBackgroundColorARGB, str
        )
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_BoldStyle, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_ItalicStyle, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_UnderlineStyle, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_FontFamilyName, str)
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupAttribute_GenericFontFamilyName, str
        )
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_Default, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_Serif, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_SansSerif, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_Monospace, str)
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupGenericFontName_ProportionalSerif, str
        )
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupGenericFontName_ProportionalSansSerif, str
        )
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupGenericFontName_MonospaceSerif, str
        )
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupGenericFontName_MonospaceSansSerif, str
        )
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_Casual, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_Cursive, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_Fantasy, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupGenericFontName_SmallCapital, str)
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupAttribute_BaseFontSizePercentageRelativeToVideoHeight,
            str,
        )
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_RelativeFontSize, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_VerticalLayout, str)
        self.assertIsInstance(CoreMedia.kCMTextVerticalLayout_LeftToRight, str)
        self.assertIsInstance(CoreMedia.kCMTextVerticalLayout_RightToLeft, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_Alignment, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAlignmentType_Start, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAlignmentType_Middle, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAlignmentType_End, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAlignmentType_Left, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupAlignmentType_Right, str)
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupAttribute_TextPositionPercentageRelativeToWritingDirection,
            str,
        )
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupAttribute_OrthogonalLinePositionPercentageRelativeToWritingDirection,
            str,
        )
        self.assertIsInstance(
            CoreMedia.kCMTextMarkupAttribute_WritingDirectionSizePercentage, str
        )
        self.assertIsInstance(CoreMedia.kCMTextMarkupAttribute_CharacterEdgeStyle, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupCharacterEdgeStyle_None, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupCharacterEdgeStyle_Raised, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupCharacterEdgeStyle_Depressed, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupCharacterEdgeStyle_Uniform, str)
        self.assertIsInstance(CoreMedia.kCMTextMarkupCharacterEdgeStyle_DropShadow, str)
